require 'perlin.rb'
require 'enumerator'

class Terrain
  attr_reader :view_node, :index
  
  def initialize(space)
    @space = space
    @index = 0
    
    @view_node = AS::Scene::GroupNode.new

    @blocks = [TerrainBlock.new(-1), TerrainBlock.new(0), TerrainBlock.new(1)]
    @blocks.each{|block| add_block(block)}
    set_offsets
    @space.rehash_static
  end
  
  def add_block(block)
    block.add_to_space(@space)
    @view_node << block.view_node
  end
  
  def remove_block(block)
    block.remove_from_space(@space)
    @view_node.delete(block.view_node)
  end
  
  def set_offsets
    @blocks[0].body.p = vec2(-TerrainBlock::Width, 0.0)
    @blocks[1].body.p = vec2(0.0, 0.0)
    @blocks[2].body.p = vec2(TerrainBlock::Width, 0.0)
  end
  
  def get_offset
    @index*TerrainBlock::Width
  end
  
  def shift_back
    remove_block(@blocks.pop)
  
    @index -= 1
    block = TerrainBlock.new(@index - 1)
    @blocks.unshift(block)
    add_block(block)
    
    set_offsets
    @space.rehash_static
  end
  
  def shift_forward
    remove_block(@blocks.shift)
  
    @index += 1
    block = TerrainBlock.new(@index + 1)
    @blocks.push(block)
    add_block(block)
    
    set_offsets
    @space.rehash_static
  end
end

class TerrainBlock
  attr_reader :view_node, :body
  
  NumSegs = 16
  Width = 160.0 
  Coef = Float(Width)/NumSegs
  
  Bevel = 3.0
  
  Tex = AS::Texture.new(AS.load_img('gfx/grass.png'), :blocky, :rgba, true)
  
  def self.terrain_func(t)
    Perlin.sample(t/1500.0)*700.0 + Perlin.sample(t/700.0)*300.0 + Perlin.sample(t/250.0)*100.0
  end

  def initialize(index)
    inf = 1.0/0.0
    @body = CP::Body.new(inf, inf)
    
    t_off = Width*index
    points = (0..NumSegs).map do|i|
      t = i*Coef
      vec2(t, TerrainBlock.terrain_func(t + t_off))
    end
    
    @segments = []
    points.each_cons(2) do|a, b|
      seg = CP::Shape::Segment.new(@body, a, b, Bevel)
      seg.e = 1.0
      seg.u = 1.0
      
      @segments << seg
    end
    
    @view_node = AS::Scene::BodyNode.new(@body) do
      AS::Scene::TextureNode.new(Tex) do
        AS::Scene::DisplayListNode.new do
          AS::DisplayList.new do
            GL.Color3f(0.3, 0.3, 0.3)
            GL.render_block(GL::QUAD_STRIP) do
              points.each do|v|
                x, y = *v.to_a
                x_off = x + t_off
                GL.TexCoord2f(x_off/256.0, 0.3); GL.Vertex2f(x, y)
                GL.TexCoord2f(x_off/256.0, 0.3); GL.Vertex2f(x, y - 300)
              end
            end
            
            GL.Color3f(1.0, 1.0, 1.0)
            GL.render_block(GL::QUAD_STRIP) do
              points.each do|v|
                x, y = *v.to_a
                x_off = x + t_off
                GL.TexCoord2f(x_off/32.0, 0.0); GL.Vertex2f(x, y - 5.0)
                GL.TexCoord2f(x_off/32.0, 0.9375); GL.Vertex2f(x, y + 10.0)
              end
            end
          end
        end
      end
    end
  end
  
  def add_to_space(space)
    @segments.each{|seg| space.add_static_shape(seg)}
  end
  
  def remove_from_space(space)
    @segments.each{|seg| space.remove_static_shape(seg)}
  end
  
  def shift(offset)
    v = vec2(offset, 0.0)
    @body.p += v
  end
end